from typing import Optional

from huibiao_framework.execption.execption import HuiBiaoException


class HuiZeModelException(HuiBiaoException):
    pass


class HuiZeModelResponseFormatError(HuiZeModelException):
    def __init__(self, msg: str):
        super().__init__(f"模型返回结果错误，报错 {msg}")


class HuiZeModelResponseCodeError(HuiZeModelException):
    def __init__(self, code: Optional[int], msg: Optional[str]):
        self.code = code
        self.msg = msg
        super().__init__(f"模型处理失败,code={self.code},msg={self.msg}!")

class LLMException(HuiBiaoException):
    def __init__(self, msg: str):
        super().__init__(f"LLM服务请求报错: {msg}")

class Qwen32bAwqException(LLMException):
    pass

class MindleDs32bAwqException(LLMException):
    pass


from typing import List, Optional, Union
from pydantic import BaseModel, Field


# region 入库
class AddDocDto(BaseModel):
    class DocDataObj(BaseModel):
        UserName: str
        DbName: str
        FileID: Union[int, str]
        FileName: str

    class ChunkData(BaseModel):
        Question: str
        Answer: str

    DocType: str = "qa"
    DocData: DocDataObj
    ChunkDatas: List[ChunkData] = []


class AddDocVo(BaseModel):
    """添加QA响应VO模型"""

    Status: str
    ChunkIds: List[int]

    def success(self) -> bool:
        return self.Status == "success"


# endregion


# region 搜索


class SearchQaDto(BaseModel):
    class DocData(BaseModel):
        UserName: str
        DbName: str

    DocType: Optional[str] = "qa"
    DocDatas: List[DocData] = []
    Query: Optional[str] = None
    Limit: Optional[int] = 100
    SearchLimit: Optional[int] = 100


class SearchQaVo(BaseModel):
    """搜索响应VO模型"""

    class MatchItem(BaseModel):
        """匹配项模型"""

        Answer: Optional[str] = None
        AnswerType: Optional[str] = None
        DbName: Optional[str] = None
        FileId: Optional[str] = None
        FileName: Optional[str] = None
        From: Optional[str] = None
        Id: Optional[int] = None
        Question: Optional[str] = None
        Score: Optional[float] = None
        SimilarFlag: Optional[int] = None
        UserName: Optional[str] = Field(..., description="用户名")

    Count: Optional[int] = Field(..., description="匹配结果数量")
    MatchList: Optional[List[MatchItem]] = Field(..., description="匹配结果列表")


# endregion



from pydantic import BaseModel, Field
from typing import List, Any, Tuple, Optional


class Line(BaseModel):
    """单个文本行的 OCR 识别结果"""

    poly: List[Tuple[int | float, int | float]] = Field(
        list,
        min_items=4,
        max_items=4,
        description="区域的四边形边界坐标，包含4个(x,y)顶点",
    )
    text: Optional[str] = None  # 识别的文本内容
    type: Optional[str] = None  # 识别类型（如 img_ocr）
    score: float | List[Any] = Field(
        None, description="识别置信度相关数据;新版是float,旧版是list"
    )


class ImageOcrVo(BaseModel):
    """OCR 模型的整体识别结果"""

    version: str = ""  # 模型版本号
    width: int | float = 0  # 图片宽度
    height: int | float = 0  # 图片高度
    angle: int | float = 0  # 图片旋转角度
    lines: List[Line] = []  # 所有识别到的文本行列表



from abc import ABC, abstractmethod


class AbstractMetricCollector(ABC):
    """指标收集器抽象基类"""

    def __init__(self, interval_sec: int = None):
        self.collect_interval = interval_sec or 30  # 收集间隔，秒

    @abstractmethod
    async def collect(self):
        """收集指标数据"""
        pass

    def set_interval(self, interval_sec: int):
        """设置收集间隔"""
        self.collect_interval = interval_sec



from pydantic import BaseModel, Field, field_validator
from typing import Generic, TypeVar, Optional

from huibiao_framework.backend import CommonStatusCode
from huibiao_framework.backend.error_code_manage import ApiErrorCodeManage
from huibiao_framework.backend.status_code import BasicStatusCode

T = TypeVar("T")


class BaseRespVo(BaseModel, Generic[T]):
    code: int
    message: str
    result: Optional[T] = Field(None, description="结果")
    requestId: Optional[str] = None

    @classmethod
    def success(cls, result: Optional[T]):
        return cls(
            code=CommonStatusCode.SUCCESS.code,
            result=result,
            message=CommonStatusCode.SUCCESS.msg,
        )

    @classmethod
    def from_status_code(cls, http_status: BasicStatusCode, **kwargs):
        return cls(
            code=http_status.code,
            message=http_status.msg.format(**kwargs),
            result=None,
        )

    def set_request_id(self, request_id: Optional[str]) -> "BaseRespVo":
        self.requestId = request_id
        return self

    @field_validator("code", mode="after")
    @classmethod
    def validate_code_prefix(cls, v: int):
        """
        对于失败的状态码，添加服务号码前缀
        """
        if v < 0:
            raise ValueError("code 必须是非负数")

        v = ApiErrorCodeManage.add_service_prefix(v)

        return v



from loguru import logger
import os
import socket

from huibiao_framework.backend.log_utils import LogSetup, LogCleaner
from huibiao_framework.config.config import LogConfig


class ServiceLoggerManager:
    """
    容器内日志目录默认是：/logger/服务名
    """

    @classmethod
    def service_log_dir(cls, service_name: str):
        return os.path.join(LogConfig.LOGGER_PATH, service_name)

    @classmethod
    def setup_log(cls, service_name: str):
        """
        日志设置
        """

        LogSetup.rotate_daily(
            log_dir=cls.service_log_dir(service_name),
            service_name=cls.add_container_id(service_name=service_name),
            add_pid_suffix=True,
            save_info=LogConfig.SAVE_INFO_LEVEL,
            save_debug=LogConfig.SAVE_DEBUG_LOG,
        )

        for pkg in LogConfig.get_disable_log_pkg():
            # 忽略一些包的日志
            logger.debug(f"ignore log: {pkg}")
            logger.disable(pkg)

    @classmethod
    def run_log_cleaner(cls, service_name: str):
        """
        启动定时任务清理日志
        """
        LogCleaner.schedule_run(
            log_dir=cls.service_log_dir(service_name=service_name), retention_day=LogConfig.LOG_RETENTION_DAY
        )

    @classmethod
    def add_container_id(cls, service_name: str):
        if not LogConfig.LOG_ADD_CONTAINED_ID:
            logger.info("日志名不增加containerId")
            return service_name

        try:
            socket_hostname = f"-{socket.gethostname()}"
        except:
            socket_hostname = ""
        if service_name in socket_hostname:
            return service_name
        else:
            return f"{service_name}{socket_hostname}"



from functools import wraps
from typing import Optional, List, Callable, Awaitable
from prometheus_client import Histogram, Counter
import os
from loguru import logger
import time

from huibiao_framework.config.config import MetricsConfig


class RequestOperationMetrics:
    """
    如果启动prometheus则使用此类
    """

    def __init__(self, buckets: Optional[List[float]] = None):
        default_buckets = [2, 5, 10, 30, 60, 120, 180, 240, 300]
        self.buckets = buckets if buckets is not None else default_buckets
        self.pid = os.getpid()

    def add(self, key, cost): ...

    def add_error(self, key): ...


class PrometheusApiMetrics(RequestOperationMetrics):
    """
    算法接口中算法操作的耗时统计
    指标名如下：
    operation_duration_seconds_count
    operation_duration_seconds_bucket
    operation_errors_total
    """

    def __init__(self, buckets: Optional[List[float]] = None):
        super().__init__(buckets=buckets)

        self.ops_duration = Histogram(
            "operation_duration_seconds",
            "Api operation duration in seconds",
            ["operation", "pid"],
            buckets=self.buckets,
        )
        self.ops_error_cnt = Counter(
            "operation_errors_total",
            "The total number of errors in operations.",
            ["operation", "pid"],
        )

    def add(self, key, cost):
        self.ops_duration.labels(operation=key, pid=self.pid).observe(cost)

    def add_error(self, key):
        self.ops_error_cnt.labels(operation=key, pid=self.pid).inc(1)


class PrometheusClientMetrics(RequestOperationMetrics):
    """
    统计client请求的耗时
    """

    def __init__(self, buckets: Optional[List[float]] = None):
        super().__init__(buckets=buckets)

        self.duration = Histogram(
            "client_duration_seconds",
            "Client response duration in seconds",
            ["client", "pid"],
            buckets=self.buckets,
        )
        self.error_cnt = Counter(
            "client_errors_total",
            "The total number of errors in client response.",
            ["client", "pid"],
        )

    def add(self, key, cost):
        self.duration.labels(client=key, pid=self.pid).observe(cost)

    def add_error(self, key):
        self.error_cnt.labels(client=key, pid=self.pid).inc(1)


class PrometheusTimeCostMetricSetting:
    __default_metrics = RequestOperationMetrics()
    __api_metrics = None
    __client_metrics = None

    @classmethod
    def initialize(
        cls,
        *,
        api_metrics_buckets: Optional[List[float]] = None,
        client_metrics_buckets: Optional[List[float]] = None,
    ):
        if (
            cls.__api_metrics is None
            and not MetricsConfig.IGNORE_REQUEST_OPERATION_METRICS
        ):
            cls.__api_metrics = PrometheusApiMetrics(buckets=api_metrics_buckets)
            logger.info(f"apiRequestOperationBuckets => {api_metrics_buckets}")

        if (
            cls.__client_metrics is None
            and not MetricsConfig.IGNORE_CLIENT_OPERATION_METRICS
        ):
            cls.__client_metrics = PrometheusClientMetrics(
                buckets=client_metrics_buckets
            )
            logger.info(f"clientRequestOperationBuckets => {client_metrics_buckets}")

    @classmethod
    def api_metrics_instance(cls) -> RequestOperationMetrics:
        return cls.__api_metrics or cls.__default_metrics

    @classmethod
    def client_metrics_instance(cls) -> RequestOperationMetrics:
        return cls.__client_metrics or cls.__default_metrics

    @classmethod
    def client_metrics_handler(cls, client_name: str):
        def decorator(func: Callable[..., Awaitable]) -> Callable[..., Awaitable]:
            @wraps(func)
            async def wrapper(*args, **kwargs):
                try:
                    start = time.perf_counter()
                    result = await func(*args, **kwargs)
                    cost = time.perf_counter() - start
                    cls.client_metrics_instance().add(client_name, cost)
                    return result
                except Exception as e:
                    cls.client_metrics_instance().add_error(client_name)
                    raise e

            return wrapper

        return decorator



from typing import List, Optional, Any

from pydantic import BaseModel, field_validator


class HuizeQwen32bAwqDto(BaseModel):
    class Message(BaseModel):
        content: str
        role: str = "user"

    Action: str = "NormalChat"
    Messages: List[Message] = []


class HuizeQwen32bAwqVo(BaseModel):
    Output: Optional[str]
    TokenProbs: Optional[List[float]]

    @field_validator("Output")
    @classmethod
    def check_output_not_empty(cls, v: str) -> str:
        if v is None or not v.strip():
            raise ValueError("Output cannot be empty")
        return v


class MindleDs32bAwqDto(BaseModel):
    class Message(BaseModel):
        role: str = "user"
        content: str  # 消息内容

    model: str = "huize"
    messages: List[Message] = []


class MindleDs32bAwqVo(BaseModel):
    id: Optional[str] = None
    choices: List["Choice"] = []
    created: Optional[int] = None
    model: Optional[str] = None
    object: Optional[Any] = None
    usage: Optional["Usage"] = None

    class Choice(BaseModel):
        finish_reason: Optional[str] = None
        index: Optional[int] = None
        message: Optional["Message"] = None
        logprobs: Optional[dict] = None

        class Message(BaseModel):
            role: Optional[str] = None
            content: Optional[str] = None
            function_call: Optional[dict] = None
            tool_calls: Optional[List[dict]] = None

    class Usage(BaseModel):
        prompt_tokens: int
        total_tokens: int
        completion_tokens: int



from .llm import HuiZeQwen32bQwqClient
from .minio_client import MinIOClient
from .embed_client import EmbedClient
from .chunk_client import ChunkClient
from .qa_search import KnowledgeQaSearchClient
from .qa_search_new import KnowledgeQaSearchClientNew
from .bid_backend_client import BidBackendFileParseClient

__all__ = [
    "MinIOClient",
    "HuiZeQwen32bQwqClient",
    "EmbedClient",
    "ChunkClient",
    "KnowledgeQaSearchClient",
    "KnowledgeQaSearchClientNew",
    "BidBackendFileParseClient",
]



import sys
from typing import Optional

from loguru import logger
import os


class LogSetup:
    FORMAT = (
        "<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> | "
        "<level>{level: <8}</level> | "
        "<cyan>p-{process}</cyan> | "
        "<cyan>t-{thread}</cyan> | "
        "<cyan>{thread.name}</cyan> | "
        "<cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> - "
        "<level>{message}</level>"
    )

    __IS_SET_ROTATE = False

    @classmethod
    def rotate_daily(
        cls,
        *,
        log_dir: str,
        service_name: str,
        add_pid_suffix: bool = True,
        save_info: bool = True,
        save_debug: bool = True,
        stderr_colorize: bool = True,
        run_id_suffix: Optional[str] = None,
    ):
        """
        日志输出终端和落盘
        : params: log_dir: 日志目录
        : params: service_name: 服务名
        : params: add_pid_suffix: 是否添加进程ID后缀
        : params: save_info: 是否保存INFO级别日志
        : params: save_debug: 是否保存DEBUG级别日志
        : params: stderr_colorize: 是否启用终端颜色显示
        : params: run_id_suffix: 运行ID后缀，用于避免多副本时的冲突
        """
        if cls.__IS_SET_ROTATE:
            return

        logger.remove()  # 清空设置，防止重复

        os.makedirs(log_dir, exist_ok=True)
        pid_suffix = f"_{os.getpid()}" if add_pid_suffix else ""
        run_id_suffix = f"_r{run_id_suffix}" if run_id_suffix else ""

        # 添加终端处理器（控制台输出）
        logger.add(
            sink=sys.stderr,  # 输出到标准错误流
            level="DEBUG",  # 终端显示更详细的DEBUG日志
            format=cls.FORMAT,
            colorize=stderr_colorize,  # 启用颜色显示
            backtrace=True,  # 堆栈信息显示在终端
        )

        if save_info:
            # 配置 INFO 及以上级别日志
            logger.add(
                os.path.join(
                    log_dir, f"{service_name}_info{pid_suffix}{run_id_suffix}.log"
                ),
                rotation="1 day",  # 每日滚动
                filter=lambda record: record["level"].no >= 20,
                format=cls.FORMAT,
                enqueue=True,
            )

        if save_debug:
            # 配置 DEBUG 级别日志
            logger.add(
                os.path.join(
                    log_dir, f"{service_name}_debug{pid_suffix}{run_id_suffix}.log"
                ),
                rotation="1 day",  # 每日滚动
                level="DEBUG",
                filter=lambda record: record["level"].no >= 10,
                format=cls.FORMAT,
                enqueue=True,
            )

        logger.info("日志设置完成")
        cls.__IS_SET_ROTATE = True



import os

from .common import AbstractMetricCollector
from prometheus_client import Gauge
import gc
from loguru import logger


class PythonGcMetricsCollector(AbstractMetricCollector):
    def __init__(self, interval_sec: int = 30):
        super().__init__(interval_sec=interval_sec)
        self.pid = os.getpid()
        # GC 总次数
        self.gc_collections_total = Gauge(
            "app_python_gc_collections_total",
            "Total number of GC collections",
            ["generation", "pid"],
        )
        # 累计回收的对象总数
        self.gc_collected_total = Gauge(
            "app_python_gc_collected_total",
            "Total objects collected",
            ["generation", "pid"],
        )
        # 累计无法回收的对象总数
        self.gc_uncollectable_total = Gauge(
            "app_python_gc_uncollectable_total",
            "Total uncollectable objects",
            ["generation", "pid"],
        )

    async def collect(self):
        try:
            # 获取当前统计信息
            stats = gc.get_stats()

            # 更新指标
            for gen, stat in enumerate(stats):
                self.gc_collections_total.labels(generation=gen, pid=self.pid).set(
                    stat.get("collections", 0)
                )
                self.gc_collected_total.labels(generation=gen, pid=self.pid).set(
                    stat.get("collected", 0)
                )
                self.gc_uncollectable_total.labels(generation=gen, pid=self.pid).set(
                    stat.get("uncollectable", 0)
                )

        except Exception as e:
            logger.error(f"Error collecting GC metrics: {e}")


class PythonGcObjectMetricsCollector(AbstractMetricCollector):
    def __init__(self, interval_sec: int = 30):
        super().__init__(interval_sec=interval_sec)
        self.pid = os.getpid()
        # 当前时刻回收的对象数
        self.gc_objects_count = Gauge(
            "app_python_gc_objects_count",
            "Current number of objects",
            ["generation", "pid"],
        )

    async def collect(self):
        try:
            counts = gc.get_count()
            for gen, count in enumerate(counts):
                self.gc_objects_count.labels(generation=gen, pid=self.pid).set(count)

        except Exception as e:
            logger.error(f"Error collecting GC Object metrics: {e}")



from typing import List, Optional, TypeVar

from pydantic import BaseModel, Field


class QAPair(BaseModel):
    """QA对模型，表示问题和答案的组合"""

    Question: str = Field(..., description="问题")
    Answer: str = Field(..., description="答案")
    SimilarFlag: Optional[int] = Field(
        default=None, description="相似标志位，0表示不相似，1表示相似，此参数可以忽略"
    )


class AddQARequest(BaseModel):
    """添加QA请求模型"""

    UserName: str = Field(..., description="用户名，不能为空")
    DbName: str = Field(..., description="问答库名（分类名），不能为空")
    FileName: str = Field(..., description="文件名称，可重名")
    FileID: int = Field(..., description="文件ID，建议保证唯一性")
    QA: List[QAPair] = Field(..., description="QA对列表")


class SearchQARequest(BaseModel):
    """
    搜索请求参数模型
    """

    UserName: List[str] = Field(..., description="用户名。不能为空。")
    DbName: List[str] = Field(
        ...,
        description="问答向量库名（分类名）。可支持多个库的搜索。列表中向量库和用户名必须一一对应的关系。",
    )
    Query: str = Field(..., description="提问内容。")
    Limit: Optional[int] = Field(
        None,
        description="TopK 个 QA 对。NeedReRank生效时为 ReRank TopK。默认值为5。",
    )
    SearchLimit: Optional[int] = Field(
        None, description="检索的TopK个结果。仅NeedReRank生效时可用。默认值为15。"
    )
    NeedReRank: Optional[bool] = Field(
        None,
        description="是否使用ReRanker重排。默认值为False，不进行重拍。若该值设为True，则进行重排。",
    )


T = TypeVar("T")


class AddQARespVo(BaseModel):
    """添加QA响应VO模型"""

    UserName: str = Field(..., description="用户名")
    DbName: str = Field(..., description="问答库名（分类名）")
    FileName: str = Field(..., description="文件名称")
    FileID: int = Field(..., description="文件ID")
    QA: List[QAPair] = Field(..., description="QA对列表")
    QAID: List[int] = Field(..., description="QA ID列表")
    Status: str = Field(..., description="状态,success表示成功")

    def success(self) -> bool:
        return self.Status == "success"


class SearchAqRespVo(BaseModel):
    """搜索响应VO模型"""

    class MatchItem(BaseModel):
        """匹配项模型"""

        FileID: Optional[int] = Field(..., description="文件ID")
        FileName: Optional[str] = Field(..., description="文件名称")
        QAID: Optional[int] = Field(..., description="QA ID")
        Question: Optional[str] = Field(..., description="问题")
        Answer: Optional[str] = Field(..., description="答案")
        SimilarFlag: Optional[int] = Field(
            ..., description="相似标志位，0表示不相似，1表示相似"
        )
        Score: Optional[float] = Field(..., description="匹配分数")
        UserName: Optional[str] = Field(..., description="用户名")
        DbName: Optional[str] = Field(..., description="问答库名（分类名）")
        From: Optional[str] = Field(..., description="来源")

    Count: Optional[int] = Field(..., description="匹配结果数量")
    MatchList: Optional[List[MatchItem]] = Field(..., description="匹配结果列表")



import os
from multiprocessing.context import assert_spawning


class ConstantClassMeta(type):
    """
    对子类也生效的元类
    """

    def __setattr__(cls, name, value):
        # 检查属性是否已存在且非特殊属性
        if hasattr(cls, name) and not name.startswith("__"):
            raise AttributeError(
                f"The property '{name}' of the constant class {cls.__name__} is not allowed to be modified."
            )

    def __call__(cls, *args, **kwargs):
        raise TypeError(f"The constant class {cls.__name__} cannot be instantiated.")


class ConstantClass(metaclass=ConstantClassMeta):
    pass


class OsAttrMeta(type(ConstantClass), type):
    def __new__(cls, name, bases, attrs):
        for attr in attrs["__annotations__"]:
            raw_value = os.getenv(attr.upper(), attrs.get(attr, None))
            expected_type = attrs["__annotations__"][attr]

            # 根据类型提示转换值
            if expected_type is bool and isinstance(raw_value, str):
                # 将 "false" 转换为 False，"true" 转换为 True
                raw_value = raw_value.lower()
                if raw_value == "true":
                    raw_value = True
                elif raw_value == "false":
                    raw_value = False
                else:
                    raise ValueError(
                        f"Invalid value for boolean attribute '{attr}': {raw_value}"
                    )
            elif expected_type is int:
                raw_value = int(raw_value)
            elif expected_type is float:
                raw_value = float(raw_value)

            # 如果环境变量不存在，则保持默认值（如果存在）
            attrs[attr] = raw_value if raw_value is not None else attrs.get(attr)

        return super().__new__(cls, name, bases, attrs)



import uuid

from fastapi import Header
from starlette.requests import Request


async def get_request_id(x_request_id: str = Header(..., alias="x-request-id")):
    """
    使用示例

    @app.post("/example/help")
    async def run(body: Body, reqid=Depends(get_request_id)):
        ...

    如果请求头中没有x-request-id，则会触发validate_interceptor的检验
    """
    return x_request_id


def gen_random_request_id():
    return str(uuid.uuid4())


def extract_reqid(reqeust: Request):
    return reqeust.headers.get("x-request-id", None)


def extract_url(reqeust: Request):
    try:
        url = reqeust.url.path
    except Exception:
        url = ""

    return url



import base64

import aiofiles


class ImageUtils:
    @classmethod
    async def encode_base64(cls, image_path) -> str:
        """
        异步读取图片文件并转换为Base64编码字符串（urlsafe格式）

        参数:
            image_path: 图片文件的路径

        返回:
            编码后的Base64字符串
        """
        # 使用aiofiles进行异步文件读取
        async with aiofiles.open(image_path, "rb") as f:
            # 异步读取文件内容
            file_content = await f.read()

        # 进行url安全的Base64编码并转换为字符串
        encoded_bytes = base64.urlsafe_b64encode(file_content)
        encoded_string = encoded_bytes.decode("utf-8")

        return encoded_string



import asyncio
import functools
from typing import Callable, Any, Union, Tuple, Type
from loguru import logger


def base_retry(
    func=None,
    *,
    name: str = None,
    delays_sec: Union[list, tuple, float, int] = 1,
    exceptions: Union[Type[Exception], Tuple[Type[Exception], ...]] = Exception,
    ignore_exceptions: Union[Type[Exception], Tuple[Type[Exception], ...]] = None
) -> Callable:
    """
    异步重试装饰器，专门用于装饰异步函数
    支持自定义重试次数（由 delays 列表长度决定）、时间间隔和异常类型
    """

    def decorator(func: Callable) -> Callable:
        @functools.wraps(func)
        async def wrapper(*args: Any, **kwargs: Any) -> Any:
            # 处理 delays 参数
            if isinstance(delays_sec, (int, float)):
                # 如果是单个数值，创建包含该值的列表
                delay_list = [delays_sec]
            else:
                # 如果是列表或元组，直接使用
                delay_list = list(delays_sec)

            func_name = name or func.__name__

            idx = 0
            while True:
                try:
                    return await func(*args, **kwargs)
                except exceptions as e:
                    if ignore_exceptions and isinstance(e, ignore_exceptions):
                        logger.info(
                            f"Ignoring exception [{type(e).__name__}] in [{func_name}]"
                        )
                        raise

                    last_exception = e

                    if idx < len(delay_list):
                        delay = delay_list[idx]
                        logger.warning(
                            f"Attempt [{func_name}] {idx + 1} failed. Retrying in {delay} seconds..."
                        )
                        await asyncio.sleep(delay)
                        idx += 1
                    else:
                        logger.error(
                            f"Attempt [{func_name}] {idx + 1} total. No more retries."
                        )
                        break

            raise last_exception

        return wrapper

    return decorator if func is None else decorator(func)






import asyncio
from typing import  TypeVar

from huibiao_framework.utils.file_manage import AbstractAsyncFile

from loguru import logger

T = TypeVar("T", bound=AbstractAsyncFile)


class TempAsyncFileWrapper:
    def __init__(self, async_file: T, is_clean: bool=True):
        self.file: T = async_file
        self.is_clean = is_clean

    async def __aenter__(self) -> T:
        return self.file

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.is_clean:
            async def delete_file():
                try:
                    self.file.delete()
                except Exception as e:
                    logger.warning(
                        f"delete file {self.file.local_path} error, {e}"
                    )
            asyncio.create_task(delete_file())





from dotenv import load_dotenv

load_dotenv(".env")

from .config import MinioConfig, ClientConfig, ModelConfig, OcrDocumentParseClientConfig, LogConfig, ServiceConfig

__all__ = ["MinioConfig", "ClientConfig", "ModelConfig", "OcrDocumentParseClientConfig", "LogConfig", "ServiceConfig"]



from starlette.requests import Request

from fastapi.exceptions import RequestValidationError
from fastapi.responses import JSONResponse
from loguru import logger

from huibiao_framework.backend import CommonStatusCode, BaseRespVo
from huibiao_framework.backend.utils import extract_url, extract_reqid


def validate_interceptor(request: Request, exc: RequestValidationError) -> JSONResponse:
    """
    拦截接口关于schema检验的报错，并返回约定的body

    当router定义的接口定‘request_id: str = Depends(get_request_id)’时，才会触发本函数的检验
    """

    # 检验请求头
    reqid: str = extract_reqid(request)
    url: str = extract_url(request)

    logger.error(f"Reqid: {reqid} | 请求体参数错误: {str(exc)[:400]}...")
    if int(request.headers.get("content-length")) == 0 and request.method in (
        "POST",
        "PUT",
        "PATCH",
    ):
        vo = BaseRespVo.from_status_code(CommonStatusCode.BODY_EMPTY_ERR, url=url)
    else:
        reason: str = parse_request_validation_error(exc)
        vo = BaseRespVo.from_status_code(
            CommonStatusCode.PARAM_ERROR, url=url, msg=reason
        )

    # 注入reqid
    vo.set_request_id(reqid)

    return JSONResponse(
        status_code=200,  # 或其他合适的 HTTP 状态码
        content=vo.model_dump(),  # 将 Pydantic 模型转换为字典
    )


def parse_request_validation_error(exc: RequestValidationError):
    """
    报错润色
    RequestValidationError([{'type': 'string_type', 'loc': ('body', 'messages', 0, 'role'), 'msg': 'Input should be a valid string...[]},
    {'type': 'string_type', 'loc': ('body', 'messages', 0, 'content'), 'msg': 'Input should be a valid string', 'input': []}])
    转换结果如下：
    '请求参数错误，发现2个字段问题：messages[0].role: Input should be a valid string；messages[0].content: Input should be a valid string'
    """
    error_fields = []
    for error in exc.errors():
        loc = error["loc"]
        # 忽略第一个元素（通常是 body）
        field_path = ".".join(
            f"[{i}]" if isinstance(i, int) else i for i in loc[1:]
        ).replace(".[", "[")
        msg = error["msg"]
        error_fields.append(f"{field_path}: {msg}")
    return f"请求参数错误，发现{len(error_fields)}个字段问题：" + "；".join(
        error_fields
    )
